/******************************************************************************
 *
 * Copyright:
 *    (C) 2000 - 2007 Embedded Artists AB
 *
 * Description:
 *    Main program for LPC2148 Education Board test program
 *
 *****************************************************************************/

#include "pre_emptive_os/api/osapi.h"
#include "pre_emptive_os/api/general.h"
#include <printf_P.h>
#include <ea_init.h>
#include <lpc2xxx.h>
#include <consol.h>
#include "uart.h"

#define PROC1_STACK_SIZE 1024
#define PROC2_STACK_SIZE 1024
#define INIT_STACK_SIZE  400

static tU8 proc1Stack[PROC1_STACK_SIZE];
static tU8 proc2Stack[PROC2_STACK_SIZE];
static tU8 initStack[INIT_STACK_SIZE];
static tU8 pid1;
static tU8 pid2;

static void proc1(void* arg);
static void proc2(void* arg);
static void initProc(void* arg);

volatile tU32 msClock;

/*****************************************************************************
 *
 * Description:
 *    The first function to execute 
 *
 ****************************************************************************/
int
main(void)
{
  tU8 error;
  tU8 pid;

  osInit();
  osCreateProcess(initProc, initStack, INIT_STACK_SIZE, &pid, 1, NULL, &error);
  osStartProcess(pid, &error);
  
  osStart();
  return 0;
}

/*****************************************************************************
 *
 * Description:
 *    A process entry function 
 *
 * Params:
 *    [in] arg - This parameter is not used in this application. 
 *
 ****************************************************************************/
static void
proc1(void* arg)
{
	for(;;)
	{
    printf("\n\n\n\n\n*******************************************************\n");
    printf("*                                                     *\n");
    printf("* This program demonstrate uart pass through          *\n");
    printf("* communication (uart#0 <-> uart#1) with              *\n");
    printf("* LPC2103 Education Board v1.0                        *\n");
    printf("*                                                     *\n");
    printf("* (C) Embedded Artists 2008                           *\n");
    printf("*                                                     *\n");
    printf("*******************************************************\n");

    //initialize uart #1: 4800bps, 8N1, no FIFO
    initUart1(B4800((FOSC * PLL_MUL) / PBSD), UART_8N1, UART_FIFO_8);

    //short delay
    uart1SendCh(0xff);
  
    //print welcome message
    uart1SendString("\n*********************************************************");
    uart1SendString("\n*                                                       *");
    uart1SendString("\n* Test program, uart pass through                       *");
    uart1SendString("\n*                                                       *");
    uart1SendString("\n* This program demonstrates a simple echo application   *");
    uart1SendString("\n* on UART #0 <-> UART #1.                               *");
    uart1SendString("\n*                                                       *");
    uart1SendString("\n*********************************************************\n");

    for(;;)
    {
      tU8 rxChar;

      //check if character received  over UART#0
      if (consolGetChar(&rxChar) == TRUE)
        uart1SendCh(rxChar);

      //check if any character has been received over UART#1
      if (uart1GetChar(&rxChar) == TRUE)
        printf("%c", rxChar);
    }
  }
}


/*****************************************************************************
 *
 * Description:
 *    A process entry function 
 *
 * Params:
 *    [in] arg - This parameter is not used in this application. 
 *
 ****************************************************************************/
static void
proc2(void* arg)
{
  while(1)  
    osSleep(50);
}


/*****************************************************************************
 *
 * Description:
 *    The entry function for the initialization process. 
 *
 * Params:
 *    [in] arg - This parameter is not used in this application. 
 *
 ****************************************************************************/
static void
initProc(void* arg)
{
  tU8 error;

  eaInit();   //initialize printf

  osCreateProcess(proc1, proc1Stack, PROC1_STACK_SIZE, &pid1, 3, NULL, &error);
  osStartProcess(pid1, &error);
//  osCreateProcess(proc2, proc2Stack, PROC2_STACK_SIZE, &pid2, 3, NULL, &error);
//  osStartProcess(pid2, &error);

  osDeleteProcess();
}

/*****************************************************************************
 *
 * Description:
 *    The timer tick entry function that is called once every timer tick
 *    interrupt in the RTOS. Observe that any processing in this
 *    function must be kept as short as possible since this function
 *    execute in interrupt context.
 *
 * Params:
 *    [in] elapsedTime - The number of elapsed milliseconds since last call.
 *
 ****************************************************************************/
void
appTick(tU32 elapsedTime)
{
  msClock += elapsedTime;
}
